package java.util.logging;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.ref.WeakReference;
import java.util.Hashtable;
import java.io.InputStream;
import java.util.Iterator;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.List;
import sun.misc.JavaAWTAccess;
import sun.misc.SharedSecrets;
import java.util.Objects;
import java.beans.PropertyChangeListener;
import sun.util.logging.PlatformLogger;
import java.security.PrivilegedExceptionAction;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.HashMap;
import java.security.Permission;
import java.lang.ref.ReferenceQueue;
import java.util.WeakHashMap;
import java.util.Map;
import java.util.Properties;

public class LogManager
{
    private static final LogManager manager;
    private volatile Properties props;
    private static final Level defaultLevel;
    private final Map<Object, Integer> listenerMap;
    private final LogManager.LoggerContext systemContext;
    private final LogManager.LoggerContext userContext;
    private volatile Logger rootLogger;
    private volatile boolean readPrimordialConfiguration;
    private boolean initializedGlobalHandlers;
    private boolean deathImminent;
    private boolean initializedCalled;
    private volatile boolean initializationDone;
    private WeakHashMap<Object, LogManager.LoggerContext> contextsMap;
    private final ReferenceQueue<Logger> loggerRefQueue;
    private static final int MAX_ITERATIONS = 400;
    private final Permission controlPermission;
    private static LoggingMXBean loggingMXBean;
    public static final String LOGGING_MXBEAN_NAME = "java.util.logging:type=Logging";
    
    protected LogManager() {
        this(checkSubclassPermissions());
    }
    
    private LogManager(Void void1) {
        this.props = new Properties();
        this.listenerMap = new HashMap();
        this.systemContext = new LogManager.SystemLoggerContext();
        this.userContext = new LogManager.LoggerContext();
        this.initializedGlobalHandlers = true;
        this.initializedCalled = false;
        this.initializationDone = false;
        this.contextsMap = null;
        this.loggerRefQueue = new ReferenceQueue();
        this.controlPermission = new LoggingPermission("control", null);
        try {
            Runtime.getRuntime().addShutdownHook(new LogManager.Cleaner());
        }
        catch (IllegalStateException ex) {}
    }
    
    private static Void checkSubclassPermissions() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("shutdownHooks"));
            securityManager.checkPermission(new RuntimePermission("setContextClassLoader"));
        }
        return null;
    }
    
    final void ensureLogManagerInitialized() {
        if (this.initializationDone || this != LogManager.manager) {
            return;
        }
        synchronized (this) {
            int n = this.initializedCalled;
            assert !this.initializationDone : "Initialization can't be done if initialized has not been called!";
            if (n || this.initializationDone) {
                return;
            }
            this.initializedCalled = true;
            try {
                AccessController.doPrivileged(new PrivilegedAction<Object>() {
                    @Override
                    public Object run() {
                        assert LogManager.this.rootLogger == null;
                        assert LogManager.this.initializedCalled && !LogManager.this.initializationDone;
                        LogManager.access$800(LogManager.this);
                        LogManager.this.rootLogger = new LogManager.RootLogger();
                        LogManager.this.addLogger(LogManager.this.rootLogger);
                        if (!LogManager.this.rootLogger.isLevelInitialized()) {
                            LogManager.this.rootLogger.setLevel(LogManager.defaultLevel);
                        }
                        LogManager.this.addLogger(Logger.global);
                        return null;
                    }
                });
            }
            finally {
                this.initializationDone = true;
            }
        }
        return;
    }
    
    public static LogManager getLogManager() {
        if (LogManager.manager != null) {
            LogManager.manager.ensureLogManagerInitialized();
        }
        return LogManager.manager;
    }
    
    private void readPrimordialConfiguration() {
        if (!this.readPrimordialConfiguration) {
            synchronized (this) {
                if (!this.readPrimordialConfiguration) {
                    if (System.out == null) {
                        return;
                    }
                    this.readPrimordialConfiguration = true;
                    try {
                        AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                            @Override
                            public Void run() throws Exception {
                                LogManager.this.readConfiguration();
                                PlatformLogger.redirectPlatformLoggers();
                                return null;
                            }
                        });
                    }
                    catch (Exception obj) {
                        assert false : "Exception raised while reading logging configuration: " + obj;
                    }
                }
            }
        }
    }
    
    @Deprecated
    public void addPropertyChangeListener(PropertyChangeListener obj) throws SecurityException {
        PropertyChangeListener propertyChangeListener = (PropertyChangeListener)Objects.requireNonNull(obj);
        this.checkPermission();
        synchronized (this.listenerMap) {
            Integer n = (Integer)this.listenerMap.get(propertyChangeListener);
            this.listenerMap.put(propertyChangeListener, Integer.valueOf((n == null) ? 1 : ((int)n + 1)));
        }
    }
    
    @Deprecated
    public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) throws SecurityException {
        this.checkPermission();
        if (propertyChangeListener != null) {
            synchronized (this.listenerMap) {
                Integer n = (Integer)this.listenerMap.get(propertyChangeListener);
                if (n != null) {
                    int intValue = (int)n;
                    if (intValue == 1) {
                        this.listenerMap.remove(propertyChangeListener);
                    }
                    else {
                        assert intValue > 1;
                        this.listenerMap.put(propertyChangeListener, Integer.valueOf(intValue - 1));
                    }
                }
            }
        }
    }
    
    private LogManager.LoggerContext getUserContext() {
        LogManager.LoggerContext value = null;
        SecurityManager securityManager = System.getSecurityManager();
        JavaAWTAccess javaAWTAccess = SharedSecrets.getJavaAWTAccess();
        if (securityManager != null && javaAWTAccess != null) {
            Object appletContext = javaAWTAccess.getAppletContext();
            if (appletContext != null) {
                synchronized (javaAWTAccess) {
                    if (this.contextsMap == null) {
                        this.contextsMap = new WeakHashMap();
                    }
                    value = (LogManager.LoggerContext)this.contextsMap.get(appletContext);
                    if (value == null) {
                        value = new LogManager.LoggerContext();
                        this.contextsMap.put(appletContext, value);
                    }
                }
            }
        }
        return (value != null) ? value : this.userContext;
    }
    
    final LogManager.LoggerContext getSystemContext() {
        return this.systemContext;
    }
    
    private List<LogManager.LoggerContext> contexts() {
        ArrayList<LogManager.LoggerContext> list = new ArrayList();
        list.add(this.getSystemContext());
        list.add(this.getUserContext());
        return list;
    }
    
    Logger demandLogger(String s, String s2, Class<?> clazz) {
        Logger logger = this.getLogger(s);
        if (logger == null) {
            Logger logger2 = new Logger(s, s2, clazz, this, false);
            while (!this.addLogger(logger2)) {
                logger = this.getLogger(s);
                if (logger != null) {
                    return logger;
                }
                continue;
            }
            return logger2;
        }
        return logger;
    }
    
    Logger demandSystemLogger(String s, String s2) {
        final Logger logger = this.getSystemContext().demandLogger(s, s2);
        do {
            if (this.addLogger(logger)) {
            }
            else {
                logger = this.getLogger(s);
            }
        } while (logger == null);
        if (logger != logger && logger.accessCheckedHandlers().length == 0) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    Handler[] accessCheckedHandlers = logger.accessCheckedHandlers();
                    for (int length = accessCheckedHandlers.length, i = 0; i < length; ++i) {
                        logger.addHandler(accessCheckedHandlers[i]);
                    }
                    return null;
                }
            });
            return logger;
        }
        return logger;
    }
    
    private void loadLoggerHandlers(final Logger logger, String s, final String s2) {
        AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                String[] access$1300 = LogManager.access$1300(LogManager.this, s2);
                for (int i = 0; i < access$1300.length; ++i) {
                    String s = access$1300[i];
                    try {
                        Handler handler = (Handler)ClassLoader.getSystemClassLoader().loadClass(s).newInstance();
                        String property = LogManager.this.getProperty(s + ".level");
                        if (property != null) {
                            Level level = Level.findLevel(property);
                            if (level != null) {
                                handler.setLevel(level);
                            }
                            else {
                                System.err.println("Can't set level for " + s);
                            }
                        }
                        logger.addHandler(handler);
                    }
                    catch (Exception obj) {
                        System.err.println("Can't load log handler \"" + s + "\"");
                        System.err.println("" + obj);
                        obj.printStackTrace();
                    }
                }
                return null;
            }
        });
    }
    
    final void drainLoggerRefQueueBounded() {
        int i = 0;
        while (i < 400) {
            if (this.loggerRefQueue == null) {
                break;
            }
            LogManager.LoggerWeakRef loggerWeakRef = (LogManager.LoggerWeakRef)this.loggerRefQueue.poll();
            if (loggerWeakRef == null) {
                break;
            }
            loggerWeakRef.dispose();
            ++i;
        }
    }
    
    public boolean addLogger(Logger logger) {
        String name = logger.getName();
        if (name == null) {
            throw new NullPointerException();
        }
        this.drainLoggerRefQueueBounded();
        if (this.getUserContext().addLocalLogger(logger)) {
            this.loadLoggerHandlers(logger, name, name + ".handlers");
            return true;
        }
        return false;
    }
    
    private static void doSetLevel(final Logger logger, final Level level) {
        if (System.getSecurityManager() == null) {
            logger.setLevel(level);
            return;
        }
        AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                logger.setLevel(level);
                return null;
            }
        });
        return;
    }
    
    private static void doSetParent(final Logger logger, final Logger parent) {
        if (System.getSecurityManager() == null) {
            logger.setParent(parent);
            return;
        }
        AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                logger.setParent(parent);
                return null;
            }
        });
        return;
    }
    
    public Logger getLogger(String s) {
        return this.getUserContext().findLogger(s);
    }
    
    public Enumeration<String> getLoggerNames() {
        return this.getUserContext().getLoggerNames();
    }
    
    public void readConfiguration() throws IOException, SecurityException {
        this.checkPermission();
        String property = System.getProperty("java.util.logging.config.class");
        if (property != null) {
            try {
                try {
                    ClassLoader.getSystemClassLoader().loadClass(property).newInstance();
                    return;
                }
                catch (ClassNotFoundException ex) {
                    Thread.currentThread().getContextClassLoader().loadClass(property).newInstance();
                    return;
                }
            }
            catch (Exception obj) {
                System.err.println("Logging configuration class \"" + property + "\" failed");
                System.err.println("" + obj);
            }
        }
        String name = System.getProperty("java.util.logging.config.file");
        if (name == null) {
            String property2 = System.getProperty("java.home");
            if (property2 == null) {
                throw new Error("Can't find java.home ??");
            }
            name = new File(new File(property2, "lib"), "logging.properties").getCanonicalPath();
        }
        try (final Object in = new FileInputStream(name)) {
            this.readConfiguration(new BufferedInputStream(in));
        }
    }
    
    public void reset() throws SecurityException {
        this.checkPermission();
        synchronized (this) {
            this.props = new Properties();
            this.initializedGlobalHandlers = true;
        }
        for (LogManager.LoggerContext loggerContext : this.contexts()) {
            Enumeration<String> loggerNames = loggerContext.getLoggerNames();
            while (loggerNames.hasMoreElements()) {
                Logger logger = loggerContext.findLogger((String)loggerNames.nextElement());
                if (logger != null) {
                    this.resetLogger(logger);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    private void resetLogger(Logger logger) {
        Handler[] handlers = logger.getHandlers();
        for (int i = 0; i < handlers.length; ++i) {
            Handler handler = handlers[i];
            logger.removeHandler(handler);
            try {
                handler.close();
            }
            catch (Exception ex) {}
        }
        String name = logger.getName();
        if (name != null && name.equals("")) {
            logger.setLevel(LogManager.defaultLevel);
        }
        else {
            logger.setLevel(null);
        }
    }
    
    private String[] parseClassNames(String s) {
        String property = this.getProperty(s);
        if (property == null) {
            return new String[0];
        }
        String trim = property.trim();
        int i = 0;
        ArrayList<String> list = new ArrayList();
        while (i < trim.length()) {
            while (i < trim.length()) {
                if (Character.isWhitespace(trim.charAt(i))) {
                    break;
                }
                if (trim.charAt(i) == ',') {
                    break;
                }
                ++i;
            }
            String substring = trim.substring(i, i);
            ++i;
            String trim2 = substring.trim();
            if (trim2.length() == 0) {
                continue;
            }
            list.add(trim2);
        }
        return (String[])list.toArray(new String[list.size()]);
    }
    
    public void readConfiguration(InputStream inStream) throws IOException, SecurityException {
        this.checkPermission();
        this.reset();
        this.props.load(inStream);
        String[] classNames = this.parseClassNames("config");
        for (int i = 0; i < classNames.length; ++i) {
            String s = classNames[i];
            try {
                ClassLoader.getSystemClassLoader().loadClass(s).newInstance();
            }
            catch (Exception obj) {
                System.err.println("Can't load config class \"" + s + "\"");
                System.err.println("" + obj);
            }
        }
        this.setLevelsOnExistingLoggers();
        Object o = null;
        synchronized (this.listenerMap) {
            if (!this.listenerMap.isEmpty()) {
                o = new HashMap(this.listenerMap);
            }
        }
        if (o != null) {
            assert !(!LogManager.Beans.isBeansPresent());
            Object propertyChangeEvent = LogManager.Beans.newPropertyChangeEvent(LogManager.class, null, null, null);
            for (Map.Entry entry : o.entrySet()) {
                Object key = entry.getKey();
                for (int intValue = (int)entry.getValue(), j = 0; j < intValue; ++j) {
                    LogManager.Beans.invokePropertyChange(key, propertyChangeEvent);
                }
            }
        }
        synchronized (this) {
            this.initializedGlobalHandlers = false;
        }
    }
    
    public String getProperty(String key) {
        return this.props.getProperty(key);
    }
    
    String getStringProperty(String s, String s2) {
        String property = this.getProperty(s);
        if (property == null) {
            return s2;
        }
        return property.trim();
    }
    
    int getIntProperty(String s, int n) {
        String property = this.getProperty(s);
        if (property == null) {
            return n;
        }
        try {
            return Integer.parseInt(property.trim());
        }
        catch (Exception ex) {
            return n;
        }
    }
    
    boolean getBooleanProperty(String s, boolean b) {
        String property = this.getProperty(s);
        if (property == null) {
            return b;
        }
        String lowerCase = property.toLowerCase();
        return lowerCase.equals("true") || lowerCase.equals("1") || (!lowerCase.equals("false") && !lowerCase.equals("0") && b);
    }
    
    Level getLevelProperty(String s, Level level) {
        String property = this.getProperty(s);
        if (property == null) {
            return level;
        }
        Level level2 = Level.findLevel(property.trim());
        return (level2 != null) ? level2 : level;
    }
    
    Filter getFilterProperty(String s, Filter filter) {
        String property = this.getProperty(s);
        try {
            if (property != null) {
                return (Filter)ClassLoader.getSystemClassLoader().loadClass(property).newInstance();
            }
        }
        catch (Exception ex) {}
        return filter;
    }
    
    Formatter getFormatterProperty(String s, Formatter formatter) {
        String property = this.getProperty(s);
        try {
            if (property != null) {
                return (Formatter)ClassLoader.getSystemClassLoader().loadClass(property).newInstance();
            }
        }
        catch (Exception ex) {}
        return formatter;
    }
    
    private synchronized void initializeGlobalHandlers() {
        if (this.initializedGlobalHandlers) {
            return;
        }
        this.initializedGlobalHandlers = true;
        if (this.deathImminent) {
            return;
        }
        this.loadLoggerHandlers(this.rootLogger, null, "handlers");
        return;
    }
    
    void checkPermission() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(this.controlPermission);
        }
    }
    
    public void checkAccess() throws SecurityException {
        this.checkPermission();
    }
    
    private synchronized void setLevelsOnExistingLoggers() {
        Enumeration<Object> propertyNames = this.props.propertyNames();
        while (propertyNames.hasMoreElements()) {
            String str = (String)propertyNames.nextElement();
            if (!str.endsWith(".level")) {
                continue;
            }
            String substring = str.substring(0, str.length() - 6);
            Level levelProperty = this.getLevelProperty(str, null);
            if (levelProperty == null) {
                System.err.println("Bad level value for property: " + str);
            }
            else {
                Iterator<LogManager.LoggerContext> iterator = this.contexts().iterator();
                while (iterator.hasNext()) {
                    Logger logger = ((LogManager.LoggerContext)iterator.next()).findLogger(substring);
                    if (logger == null) {
                        continue;
                    }
                    logger.setLevel(levelProperty);
                }
            }
        }
    }
    
    public static synchronized LoggingMXBean getLoggingMXBean() {
        if (LogManager.loggingMXBean == null) {
            LogManager.loggingMXBean = new Logging();
        }
        return LogManager.loggingMXBean;
    }
    
    static void access$800(LogManager logManager) {
        logManager.readPrimordialConfiguration();
    }
    
    static void access$1100(Logger logger, Level level) {
        doSetLevel(logger, level);
    }
    
    static void access$1200(Logger logger, Logger logger2) {
        doSetParent(logger, logger2);
    }
    
    static String[] access$1300(LogManager logManager, String s) {
        return logManager.parseClassNames(s);
    }
    
    static void access$1500(LogManager logManager) {
        logManager.initializeGlobalHandlers();
    }
    
    static {
        defaultLevel = Level.INFO;
        manager = (LogManager)AccessController.doPrivileged(new PrivilegedAction<LogManager>() {
            @Override
            public LogManager run() {
                Object o = null;
                String property = null;
                try {
                    property = System.getProperty("java.util.logging.manager");
                    if (property != null) {
                        try {
                            o = (Object)ClassLoader.getSystemClassLoader().loadClass(property).newInstance();
                        }
                        catch (ClassNotFoundException ex) {
                            o = (Object)Thread.currentThread().getContextClassLoader().loadClass(property).newInstance();
                        }
                    }
                }
                catch (Exception ex2) {
                    System.err.println("Could not load Logmanager \"" + property + "\"");
                    ex2.printStackTrace();
                }
                if (o == null) {
                    o = new LogManager();
                    return o;
                }
                return o;
            }
        });
        LogManager.loggingMXBean = null;
    }
    
    private class Cleaner extends Thread
    {
        private Cleaner() {
            this.setContextClassLoader(null);
        }
        
        @Override
        public void run() {
            LogManager f1 = LogManager.manager;
            synchronized (LogManager.this) {
                LogManager.this.deathImminent = true;
                LogManager.this.initializedGlobalHandlers = true;
            }
            LogManager.this.reset();
        }
    }
    
    class LoggerContext
    {
        private final Hashtable<String, LogManager.LoggerWeakRef> namedLoggers;
        private final LogManager.LogNode root;
        
        private LoggerContext() {
            this.namedLoggers = new Hashtable();
            this.root = new LogManager.LogNode(null, this);
        }
        
        final boolean requiresDefaultLoggers() {
            int n = this.getOwner() == LogManager.manager;
            if (!(n == 0)) {
                this.getOwner().ensureLogManagerInitialized();
            }
            return n;
        }
        
        final LogManager getOwner() {
            return LogManager.this;
        }
        
        final Logger getRootLogger() {
            return this.getOwner().rootLogger;
        }
        
        final Logger getGlobalLogger() {
            return Logger.global;
        }
        
        Logger demandLogger(String s, String s2) {
            return this.getOwner().demandLogger(s, s2, null);
        }
        
        private void ensureInitialized() {
            if (this.requiresDefaultLoggers()) {
                this.ensureDefaultLogger(this.getRootLogger());
                this.ensureDefaultLogger(this.getGlobalLogger());
            }
        }
        
        synchronized Logger findLogger(String key) {
            this.ensureInitialized();
            LogManager.LoggerWeakRef loggerWeakRef = (LogManager.LoggerWeakRef)this.namedLoggers.get(key);
            if (loggerWeakRef == null) {
                return null;
            }
            Logger logger = (Logger)loggerWeakRef.get();
            if (logger == null) {
                loggerWeakRef.dispose();
                return logger;
            }
            return logger;
        }
        
        private void ensureAllDefaultLoggers(Logger logger) {
            if (this.requiresDefaultLoggers()) {
                String name = logger.getName();
                if (!name.isEmpty()) {
                    this.ensureDefaultLogger(this.getRootLogger());
                    if (!"global".equals(name)) {
                        this.ensureDefaultLogger(this.getGlobalLogger());
                    }
                }
            }
        }
        
        private void ensureDefaultLogger(Logger logger) {
            if (!this.requiresDefaultLoggers() || logger == null || (logger != Logger.global && logger != LogManager.this.rootLogger)) {
                assert logger == null;
                return;
            }
            else {
                if (!this.namedLoggers.containsKey(logger.getName())) {
                    this.addLocalLogger(logger, false);
                }
                return;
            }
        }
        
        boolean addLocalLogger(Logger logger) {
            return this.addLocalLogger(logger, this.requiresDefaultLoggers());
        }
        
        synchronized boolean addLocalLogger(Logger logger, boolean b) {
            if (b) {
                this.ensureAllDefaultLoggers(logger);
            }
            String name = logger.getName();
            if (name == null) {
                throw new NullPointerException();
            }
            LogManager.LoggerWeakRef loggerWeakRef = (LogManager.LoggerWeakRef)this.namedLoggers.get(name);
            if (loggerWeakRef != null) {
                if (loggerWeakRef.get() == null) {
                    loggerWeakRef.dispose();
                }
                else {
                    return false;
                }
            }
            LogManager owner = this.getOwner();
            logger.setLogManager(owner);
            LogManager.LoggerWeakRef loggerWeakRef2 = owner.new LoggerWeakRef(logger);
            this.namedLoggers.put(name, loggerWeakRef2);
            Level levelProperty = owner.getLevelProperty(name + ".level", null);
            if (levelProperty != null && !logger.isLevelInitialized()) {
                LogManager.access$1100(logger, levelProperty);
            }
            this.processParentHandlers(logger, name);
            LogManager.LogNode node = this.getNode(name);
            node.loggerRef = loggerWeakRef2;
            Logger logger2 = null;
            for (LogManager.LogNode logNode = node.parent; logNode != null; logNode = logNode.parent) {
                LogManager.LoggerWeakRef loggerRef = logNode.loggerRef;
                if (loggerRef != null) {
                    logger2 = (Logger)loggerRef.get();
                    if (logger2 != null) {
                        break;
                    }
                }
            }
            if (logger2 != null) {
                LogManager.access$1200(logger, logger2);
            }
            node.walkAndSetParent(logger);
            loggerWeakRef2.setNode(node);
            return true;
        }
        
        synchronized void removeLoggerRef(String key, LogManager.LoggerWeakRef value) {
            this.namedLoggers.remove(key, value);
        }
        
        synchronized Enumeration<String> getLoggerNames() {
            this.ensureInitialized();
            return this.namedLoggers.keys();
        }
        
        private void processParentHandlers(Logger logger, String s) {
            LogManager owner = this.getOwner();
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    if (logger != owner.rootLogger && !owner.getBooleanProperty(s + ".useParentHandlers", true)) {
                        logger.setUseParentHandlers(false);
                    }
                    return null;
                }
            });
            int fromIndex = 1;
            while (true) {
                int index = s.indexOf(".", fromIndex);
                if (index < 0) {
                    break;
                }
                String substring = s.substring(0, index);
                if (owner.getProperty(substring + ".level") != null || owner.getProperty(substring + ".handlers") != null) {
                    this.demandLogger(substring, null);
                }
                fromIndex = index + 1;
            }
        }
        
        LogManager.LogNode getNode(String substring) {
            if (substring == null || substring.equals("")) {
                return this.root;
            }
            LogManager.LogNode root = this.root;
            while (substring.length() > 0) {
                int index = substring.indexOf(".");
                String substring2;
                if (index > 0) {
                    substring2 = substring.substring(0, index);
                    substring = substring.substring(index + 1);
                }
                else {
                    substring2 = substring;
                    substring = "";
                }
                if (root.children == null) {
                    root.children = new HashMap();
                }
                LogManager.LogNode value = (LogManager.LogNode)root.children.get(substring2);
                if (value == null) {
                    value = new LogManager.LogNode(root, this);
                    root.children.put(substring2, value);
                }
                root = value;
            }
            return root;
        }
    }
    
    final class SystemLoggerContext extends LogManager.LoggerContext
    {
        @Override
        Logger demandLogger(String s, String s2) {
            Logger logger = this.findLogger(s);
            if (logger == null) {
                Logger logger2 = new Logger(s, s2, null, this.getOwner(), true);
                do {
                    if (this.addLocalLogger(logger2)) {
                        logger = logger2;
                    }
                    else {
                        logger = this.findLogger(s);
                    }
                } while (logger == null);
                return logger;
            }
            return logger;
        }
    }
    
    final class LoggerWeakRef extends WeakReference<Logger>
    {
        private String name;
        private LogManager.LogNode node;
        private WeakReference<Logger> parentRef;
        private boolean disposed;
        
        LoggerWeakRef(Logger referent) {
            super(referent, LogManager.this.loggerRefQueue);
            this.disposed = false;
            this.name = referent.getName();
        }
        
        void dispose() {
            synchronized (this) {
                if (this.disposed) {
                    return;
                }
                this.disposed = true;
            }
            LogManager.LogNode node = this.node;
            if (node != null) {
                synchronized (node.context) {
                    node.context.removeLoggerRef(this.name, this);
                    this.name = null;
                    if (node.loggerRef == this) {
                        node.loggerRef = null;
                    }
                    this.node = null;
                }
            }
            if (this.parentRef != null) {
                Logger logger = (Logger)this.parentRef.get();
                if (logger != null) {
                    logger.removeChildLogger(this);
                }
                this.parentRef = null;
            }
        }
        
        void setNode(LogManager.LogNode node) {
            this.node = node;
        }
        
        void setParentRef(WeakReference<Logger> parentRef) {
            this.parentRef = parentRef;
        }
    }
    
    private static class LogNode
    {
        HashMap<String, LogManager.LogNode> children;
        LogManager.LoggerWeakRef loggerRef;
        LogManager.LogNode parent;
        final LogManager.LoggerContext context;
        
        LogNode(LogManager.LogNode parent, LogManager.LoggerContext context) {
            this.parent = parent;
            this.context = context;
        }
        
        void walkAndSetParent(Logger logger) {
            if (this.children == null) {
                return;
            }
            for (LogManager.LogNode logNode : this.children.values()) {
                LogManager.LoggerWeakRef loggerRef = logNode.loggerRef;
                Logger logger2 = (loggerRef == null) ? null : ((Logger)loggerRef.get());
                if (logger2 == null) {
                    logNode.walkAndSetParent(logger);
                }
                else {
                    LogManager.access$1200(logger2, logger);
                }
            }
            return;
        }
    }
    
    private final class RootLogger extends Logger
    {
        private RootLogger() {
            super("", null, null, LogManager.this, true);
        }
        
        @Override
        public void log(LogRecord logRecord) {
            LogManager.access$1500(LogManager.this);
            super.log(logRecord);
        }
        
        @Override
        public void addHandler(Handler handler) {
            LogManager.access$1500(LogManager.this);
            super.addHandler(handler);
        }
        
        @Override
        public void removeHandler(Handler handler) {
            LogManager.access$1500(LogManager.this);
            super.removeHandler(handler);
        }
        
        @Override
        Handler[] accessCheckedHandlers() {
            LogManager.access$1500(LogManager.this);
            return super.accessCheckedHandlers();
        }
    }
    
    private static class Beans
    {
        private static final Class<?> propertyChangeListenerClass;
        private static final Class<?> propertyChangeEventClass;
        private static final Method propertyChangeMethod;
        private static final Constructor<?> propertyEventCtor;
        
        private static Class<?> getClass(String name) {
            try {
                return Class.forName(name, true, LogManager.Beans.class.getClassLoader());
            }
            catch (ClassNotFoundException ex) {
                return null;
            }
        }
        
        private static Constructor<?> getConstructor(Class<?> clazz, Class<?>... parameterTypes) {
            try {
                return (clazz == null) ? null : clazz.getDeclaredConstructor(parameterTypes);
            }
            catch (NoSuchMethodException detailMessage) {
                throw new AssertionError(detailMessage);
            }
        }
        
        private static Method getMethod(Class<?> clazz, String name, Class<?>... parameterTypes) {
            try {
                return (clazz == null) ? null : clazz.getMethod(name, parameterTypes);
            }
            catch (NoSuchMethodException detailMessage) {
                throw new AssertionError(detailMessage);
            }
        }
        
        static boolean isBeansPresent() {
            return LogManager.Beans.propertyChangeListenerClass != null && LogManager.Beans.propertyChangeEventClass != null;
        }
        
        static Object newPropertyChangeEvent(Object o, String s, Object o2, Object o3) {
            try {
                return LogManager.Beans.propertyEventCtor.newInstance(new Object[] { o, s, o2, o3 });
            }
            catch (InstantiationException | IllegalAccessException detailMessage) {
                throw new AssertionError(detailMessage);
            }
            catch (InvocationTargetException detailMessage2) {
                Throwable cause = detailMessage2.getCause();
                if ((cause instanceof Error)) {
                    throw (Error)cause;
                }
                if ((cause instanceof RuntimeException)) {
                    throw (RuntimeException)cause;
                }
                throw new AssertionError(detailMessage2);
            }
        }
        
        static void invokePropertyChange(Object obj, Object o) {
            try {
                LogManager.Beans.propertyChangeMethod.invoke(obj, new Object[] { o });
            }
            catch (IllegalAccessException detailMessage) {
                throw new AssertionError(detailMessage);
            }
            catch (InvocationTargetException detailMessage2) {
                Throwable cause = detailMessage2.getCause();
                if ((cause instanceof Error)) {
                    throw (Error)cause;
                }
                if ((cause instanceof RuntimeException)) {
                    throw (RuntimeException)cause;
                }
                throw new AssertionError(detailMessage2);
            }
        }
        
        static {
            propertyChangeListenerClass = getClass("java.beans.PropertyChangeListener");
            propertyChangeEventClass = getClass("java.beans.PropertyChangeEvent");
            propertyChangeMethod = getMethod(LogManager.Beans.propertyChangeListenerClass, "propertyChange", new Class[] { LogManager.Beans.propertyChangeEventClass });
            propertyEventCtor = getConstructor(LogManager.Beans.propertyChangeEventClass, new Class[] { Object.class, String.class, Object.class, Object.class });
        }
    }
}
